@@ -1,6 +1,6 @@
 <?php
 	/**
-	 * Copyright (c) 2008- Samuli Järvelä
+	 * Copyright (c) 2008- Samuli Jrvel
 	 *
 	 * All rights reserved. This program and the accompanying materials
 	 * are made available under the terms of the Eclipse Public License v1.0
@@ -17,5 +17,9 @@
 	$DB_DATABASE = "mollify";	/* optional, default is "mollify" */
 	$DB_USER = "mollify";
 	$DB_PASSWORD = "mollify";
-	
+
+	$SETTINGS = array(
+		"timezone" => "Europe/Helsinki"	// change this to match your timezone
+	);
+
 ?>
\ No newline at end of file
