@@ -1,6 +1,6 @@
 <?php
 	/**
-	 * Copyright (c) 2008- Samuli Järvelä
+	 * Copyright (c) 2008- Samuli Jrvel
 	 *
 	 * All rights reserved. This program and the accompanying materials
 	 * are made available under the terms of the Eclipse Public License v1.0
@@ -11,13 +11,17 @@
 	
 	/* For configuration instructions, see ReadMe.txt or wiki page at http://code.google.com/p/mollify/wiki/Installation */
 	
-	/* File permission mode: "RW" = Read/write, "RO" = Read-only (default if omitted) */
+	/* Default permission: "RW" = Read/write, "RO" = Read-only (default if omitted) */
 	
-	$FILE_PERMISSION_MODE = "RO";
+	$DEFAULT_PERMISSION = "RO";
 	
-	$PUBLISHED_DIRECTORIES = array(
+	$PUBLISHED_FOLDERS = array(
 		"r1" => array("name" => "Folder A", "path" => "/foo/bar"),
 		"r2" => array("name" => "Folder B", "path" => "/foo/bay")
 	);
 	
+	$SETTINGS = array(
+		"timezone" => "Europe/Helsinki"	// change this to match your timezone
+	);
+	
 ?>
\ No newline at end of file
