@@ -11,12 +11,12 @@
 	 */
 
 	class MySQLConfigurationProvider extends ConfigurationProvider {
-		const VERSION = "1_5_4";
+		const VERSION = "1_6_0";
 		
 		private $db;
 		
 		public function __construct($settings) {
-			global $DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX;
+			global $DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX, $DB_CHARSET;
 			
 			if (!isset($DB_USER) or !isset($DB_PASSWORD)) throw new ServiceException("INVALID_CONFIGURATION", "No database information defined");
 			
@@ -31,7 +31,13 @@
 			
 			require_once("include/mysql/MySQLDatabase.class.php");
 			$this->db = new MySQLDatabase($host, $DB_USER, $DB_PASSWORD, $database, $tablePrefix);
+			
 			$this->db->connect();
+			if (isset($DB_CHARSET)) $this->db->setCharset($DB_CHARSET);
+		}
+		
+		public function getType() {
+			return ConfigurationProvider::TYPE_DATABASE;
 		}
 		
 		public function db() {
@@ -43,7 +49,7 @@
 		}
 
 		public function getSupportedFeatures() {
-			return array('change_password', 'description_update', 'administration', 'permission_update', 'user_groups', 'event_logging');
+			return array('change_password', 'description_update', 'administration', 'permission_update', 'user_groups');
 		}
 		
 		public function featureEnabledByDefault($name, $default) {
@@ -73,22 +79,39 @@
 			
 			return $result->firstRow();
 		}
+
+		public function getUserByName($username) {
+			$result = $this->db->query(sprintf("SELECT id, name, password FROM ".$this->db->table("user")." WHERE name='%s'", $this->db->string($username)));
+			$matches = $result->count();
+			
+			if ($matches === 0) {
+				Logging::logError("No user found with name [".$username."]");
+				return NULL;
+			}
+			
+			if ($matches > 1) {
+				Logging::logError("Duplicate user found with name [".$username."]");
+				return FALSE;
+			}
+			
+			return $result->firstRow();
+		}
 		
 		public function getAllUsers() {
-			return $this->db->query("SELECT id, name, permission_mode FROM ".$this->db->table("user")." where is_group = 0 ORDER BY id ASC")->rows();
+			return $this->db->query("SELECT id, name, email, permission_mode FROM ".$this->db->table("user")." where is_group = 0 ORDER BY id ASC")->rows();
 		}
 
 		public function getUser($id) {
-			return $this->db->query(sprintf("SELECT id, name FROM ".$this->db->table("user")." WHERE id='%s'", $this->db->string($id)))->firstRow();
+			return $this->db->query(sprintf("SELECT id, name, email FROM ".$this->db->table("user")." WHERE id='%s'", $this->db->string($id)))->firstRow();
 		}
 		
-		public function addUser($name, $pw, $permission) {
-			$this->db->update(sprintf("INSERT INTO ".$this->db->table("user")." (name, password, permission_mode, is_group) VALUES ('%s', '%s', '%s', 0)", $this->db->string($name), $this->db->string($pw), $this->db->string($permission)));
-			return TRUE;
+		public function addUser($name, $pw, $email, $permission) {
+			$this->db->update(sprintf("INSERT INTO ".$this->db->table("user")." (name, password, email, permission_mode, is_group) VALUES ('%s', '%s', %s, '%s', 0)", $this->db->string($name), $this->db->string($pw), $this->db->string($email, TRUE), $this->db->string($permission)));
+			return $this->db->lastId();
 		}
 	
-		public function updateUser($id, $name, $permission, $description = NULL) {
-			$affected = $this->db->update(sprintf("UPDATE ".$this->db->table("user")." SET name='%s', permission_mode='%s', description='%s' WHERE id='%s'", $this->db->string($name), $this->db->string($permission), $this->db->string($description), $this->db->string($id)));			
+		public function updateUser($id, $name, $email, $permission, $description = NULL) {
+			$affected = $this->db->update(sprintf("UPDATE ".$this->db->table("user")." SET name='%s', email=%s, permission_mode='%s', description='%s' WHERE id='%s'", $this->db->string($name), $this->db->string($email, TRUE), $this->db->string($permission), $this->db->string($description), $this->db->string($id)));			
 			return TRUE;
 		}
 		
@@ -152,7 +175,7 @@
 		}
 
 		public function updateUserGroup($id, $name, $description) {
-			return $this->updateUser($id, $name, NULL, $description);
+			return $this->updateUser($id, $name, NULL, NULL, $description);
 		}
 		
 		public function removeUserGroup($id) {
@@ -200,7 +223,7 @@
 
 		public function addFolder($name, $path) {
 			$this->db->update(sprintf("INSERT INTO ".$this->db->table("folder")." (name, path) VALUES ('%s', '%s')", $this->db->string($name), $this->db->string($path)));
-			return TRUE;
+			return $this->db->lastId();
 		}
 	
 		public function updateFolder($id, $name, $path) {
@@ -231,6 +254,10 @@
 			return $this->db->query(sprintf("SELECT ".$folderTable.".id, ".$userFolderTable.".name, ".$folderTable.".name as default_name, ".$folderTable.".path FROM ".$userFolderTable.", ".$folderTable." WHERE user_id='%s' AND ".$folderTable.".id = ".$userFolderTable.".folder_id", $this->db->string($userId)))->rows();
 		}
 		
+		public function addUserFolders($userId, $folderIds) {
+			foreach($folderIds as $id) $this->addUserFolder($userId, $id, NULL);
+		}
+		
 		public function addUserFolder($userId, $folderId, $name) {
 			if ($name != NULL) {
 				$this->db->update(sprintf("INSERT INTO ".$this->db->table("user_folder")." (user_id, folder_id, name) VALUES ('%s', '%s', '%s')", $this->db->string($userId), $this->db->string($folderId), $this->db->string($name)));
@@ -304,40 +331,75 @@
 		function getItemPermission($item, $userId) {
 			$table = $this->db->table("item_permission");
 			$id = $this->itemId($item);
-			$userIds = array($userId);
-			if ($this->env->session()->hasParam("groups")) {
+			
+			$userQuery = sprintf("(user_id = '%s')", $userId);
+			$userIds = array(0, $userId);
+			if ($this->env->authentication()->hasUserGroups()) {
 				foreach($this->env->authentication()->getUserGroups() as $g)
 					$userIds[] = $g['id'];
 			}
-			
 			$userQuery = sprintf("(user_id in (%s))", $this->db->arrayString($userIds));
-			$query = NULL;
-	
-			if ($item->isFile()) {
-				$parent = $item->parent();
+			
+			// order within category into 1) user specific 2) group 3) item default
+			$subcategoryQuery = sprintf("(IF(user_id = '%s', 1, IF(user_id = '0', 3, 2)))", $userId);
+
+			// item permissions
+			$query = sprintf("(SELECT permission, user_id, 1 AS 'category', %s AS 'subcategory' FROM `".$table."` WHERE item_id = '%s' AND %s)", $subcategoryQuery, $id, $userQuery);
+					
+			if ($item->isFile() or !$item->isRoot()) {
+				$parentId = $this->itemId($item->parent());
+				$rootId = $this->itemId($item->root());
 				
-				if ($parent != NULL) {
-					$parentId = $this->itemId($parent);					
-					$query = sprintf(
-						"(SELECT permission, user_id, 1 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND %s) UNION ALL ".
-						"(SELECT permission, user_id, 2 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND user_id = 0) UNION ALL ".
-						"(SELECT permission, user_id, 3 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND %s) UNION ALL ".
-						"(SELECT permission, user_id, 4 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND user_id = 0)", $id, $userQuery, $id, $parentId, $userQuery, $parentId);
+				$hierarchyQuery = "(item_id REGEXP '^".$rootId;
+				$hierarchyQueryEnd = "";
+				$parts = preg_split("/\//", substr($parentId, strlen($rootId)), -1, PREG_SPLIT_NO_EMPTY);
+				Logging::logDebug(Util::array2str($parts));
+				foreach($parts as $part) {
+					$hierarchyQuery .= "(".$part."/";
+					$hierarchyQueryEnd .= ")*";
 				}
+				$hierarchyQuery .= $hierarchyQueryEnd."$')";
+			
+				$subcategoryQuery = sprintf("(((%s - CHAR_LENGTH(item_id)) * 10) + IF(user_id = '%s', 0, IF(user_id = '0', 2, 1)))", strlen($parentId), $userId);
+				$query .= sprintf(
+					" UNION ALL (SELECT permission, user_id, 2 AS 'category', %s AS 'subcategory' FROM `".$table."` WHERE %s AND %s) ", $subcategoryQuery, $hierarchyQuery, $userQuery);
 			}
 			
-			if ($query === NULL) {
-				$query = sprintf(
-					"(SELECT permission, user_id, 1 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND %s) UNION ALL ".
-					"(SELECT permission, user_id, 2 AS 'index' FROM `".$table."` WHERE item_id = '%s' AND user_id = 0)", $id, $userQuery, $id);
-			}
+			$query = "SELECT permission FROM (".$query.") AS u ORDER BY u.category ASC, u.subcategory ASC, u.permission DESC";
 			
-			$query = "SELECT permission FROM (".$query.") AS u ORDER BY u.index ASC, u.permission DESC";
-			
-			$result = $this->db->query($query);			
+			$result = $this->db->query($query);
 			if ($result->count() < 1) return NULL;
 			return $result->value(0);
 		}
+		
+		public function getAllItemPermissions($parent, $userId) {
+			$parentId = $this->itemId($parent);
+			$table = $this->db->table("item_permission");
+			$userIds = array($userId);
+			if ($this->env->authentication()->hasUserGroups()) {
+				foreach($this->env->authentication()->getUserGroups() as $g)
+					$userIds[] = $g['id'];
+			}
+			$userIds[] = "0";
+			$userQuery = sprintf("(user_id in (%s))", $this->db->arrayString($userIds));
+
+			$itemFilter = "SELECT distinct item_id from `".$table."` where ".$userQuery." and item_id REGEXP '^".$parentId."[^/]+[/]?$'";
+			$query = sprintf("SELECT item_id, permission, (IF(user_id = '%s', 1, IF(user_id = '0', 3, 2))) as ind from `%s` where %s and item_id in (%s) order by item_id asc, ind asc, permission desc", $userId, $table, $userQuery, $itemFilter);
+			
+			$all = $this->db->query($query)->rows();
+			$all[] = array(
+				"item_id" => $parentId,
+				"permission" => $this->getItemPermission($parent, $userId)
+			);
+			$k = array();
+			$prev = NULL;
+			foreach($all as $p) {
+				$id = $p["item_id"];
+				if ($id != $prev) $k[$id] = strtoupper($p["permission"]);
+				$prev = $id;
+			}
+			return $k;
+		}
 	
 		function getItemPermissions($item) {
 			$id = $this->itemId($item);
@@ -385,6 +447,16 @@
 			$this->db->update($query);							
 			return TRUE;
 		}
+		
+		public function addItemPermission($id, $permission, $userId) {
+			$permission = $this->db->string(strtolower($permission));
+			$id = $this->db->string($id);
+			$user = $this->db->string($userId);
+
+			$query = sprintf("INSERT INTO `".$this->db->table("item_permission")."` (item_id, user_id, permission) VALUES ('%s', '%s', '%s')", $id, $user, $permission);
+			$this->db->update($query);							
+			return TRUE;
+		}
 	
 		private function updateItemPermissionValues($list) {
 			foreach($list as $item) {
@@ -433,7 +505,7 @@
 		}
 		
 		private function itemId($item) {
-			return $this->db->string($item->internalId());
+			return $this->db->string($item->id());
 		}
 	}
 ?>
\ No newline at end of file
