@@ -18,7 +18,10 @@
 		private $env;
 		private $allowedUploadTypes;
 		private $permissionCache = array();
-		private $providers = array();
+		private $folderCache = array();
+		private $detailsPlugins = array();
+		
+		public $allowFilesystems = TRUE;
 
 		function __construct($env) {
 			require_once("MollifyFilesystem.class.php");
@@ -31,21 +34,36 @@
 			FileEvent::register($this->env->events());
 		}
 		
-		public function initialize($request) {}
+		public function initialize() {}
 
-		public function registerProvider($provider) {
-			$this->providers[] = $provider;
+		public function registerDetailsPlugin($plugin) {
+			$this->detailsPlugins[] = $plugin;
 		}
-
-		public function onSessionStarted() {
-			$this->env->session()->param('folders', $this->getUserFolders());
+		
+		public function getRootFolders($all = FALSE) {
+			$list = array();
+			
+			foreach($this->getFolderDefs($all) as $folderDef) {				
+				$root = $this->filesystem($folderDef, !$all)->root();
+				if (!$this->env->authentication()->hasReadRights($this->permission($root))) continue;
+				$list[$folderDef['id']] = $root;
+			}
+			
+			return $list;
 		}
+		
+		private function getFolderDefs($all = FALSE) {
+			if ($this->env->configuration()->isAuthenticationRequired() and !$all)
+				$folderDefs = $this->env->configuration()->getUserFolders($this->env->authentication()->getUserId());
+			else
+				$folderDefs = $this->env->configuration()->getFolders();
 
-		private function getUserFolders() {
-			$folderDefs = $this->env->configuration()->getUserFolders($this->env->authentication()->getUserId());
 			$list = array();
 			
 			foreach($folderDefs as $folderDef) {
+				$root = $this->filesystem($folderDef, !$all)->root();
+				if (!$this->env->authentication()->hasReadRights($this->permission($root))) continue;
+				
 				if (!isset($folderDef["name"]) and !isset($folderDef["default_name"])) {
 					$this->env->session()->reset();
 					throw new ServiceException("INVALID_CONFIGURATION", "Folder definition does not have a name (".$folderDef['id'].")");
@@ -87,11 +105,13 @@
 			);
 			
 			$result["folders"] = array();
-			
-			foreach($this->env->session()->param('folders') as $id => $folderDef) {
+			foreach($this->getRootFolders() as $id => $folder) {
 				$result["folders"][] = array(
-					"id" => $this->publicId($id),
-					"name" => $folderDef['name'] != NULL ? $folderDef['name'] : $folderDef['default_name']
+					"id" => $folder->publicId(),
+					"name" => $folder->name(),
+					"parent_id" => NULL,
+					"root_id" => $folder->publicId(),
+					"path" => ""
 				);
 			}
 
@@ -109,33 +129,28 @@
 		}
 		
 		public function item($id, $nonexisting = FALSE) {
-			$internalId = $this->internalId($id);
-			$parts = explode(":".DIRECTORY_SEPARATOR, $internalId);
+			$parts = explode(":".DIRECTORY_SEPARATOR, $id);
 			if (count($parts) != 2) throw new ServiceException("INVALID_CONFIGURATION", "Invalid item id: ".$id);
 			
 			$filesystemId = $parts[0];
 			$path = $parts[1];
+			if (strpos($path, "..") != FALSE) throw new ServiceException("INVALID_REQUEST");
 			
-			if ($this->env->authentication()->isAuthenticated()) { 
-				$folderDefs = $this->env->session()->param('folders');
-				if (!array_key_exists($filesystemId, $folderDefs))
-					throw new ServiceException("INVALID_CONFIGURATION", "Invalid item folder: ".$id);
-				
-				$folderDef = $folderDefs[$filesystemId];
+			if (array_key_exists($filesystemId, $this->folderCache)) {
+				$folderDef = $this->folderCache[$filesystemId];
 			} else {
+				if (!$this->isFolderValid($filesystemId)) throw new ServiceException("UNAUTHORIZED");
 				$folderDef = $this->env->configuration()->getFolder($filesystemId);
-				if (!$folderDef)
-					throw new ServiceException("INVALID_CONFIGURATION", "Invalid item: ".$id);
+				$this->folderCache[$filesystemId] = $folderDef;
 			}
+			
 			return $this->filesystem($folderDef)->createItem($id, $path, $nonexisting);
 		}
 		
-		public function publicId($filesystemId, $path = "") {
-			return base64_encode($filesystemId.":".DIRECTORY_SEPARATOR.$path);
-		}
-
-		public function internalId($itemId) {
-			return base64_decode($itemId);
+		private function isFolderValid($id) {
+			if ($this->allowFilesystems or !$this->env->configuration()->isAuthenticationRequired()) return TRUE;
+			$folders = $this->getFolderDefs();
+			return array_key_exists($id, $folders);
 		}
 		
 		public function assertFilesystem($folderDef) {
@@ -155,14 +170,24 @@
 			return array('mollify.dsc', 'mollify.uac');	//TODO get from settings and/or configuration etc
 		}
 		
-		public function folders($parent) {
-			$this->assertRights($parent, Authentication::RIGHTS_READ, "folders");
-			return $parent->folders();
+		public function items($folder) {
+			//make sure folder permissions are fetched into cache
+			$this->fetchPermissions($folder);
+			$this->assertRights($folder, Authentication::RIGHTS_READ, "items");
+
+			$list = array();
+			foreach($folder->items() as $i) {
+				if (!$this->isItemVisible($i)) continue;
+				$list[] = $i;
+			}
+			return $list;
 		}
 		
-		public function files($parent) {
-			$this->assertRights($parent, Authentication::RIGHTS_READ, "files");
-			return $parent->files();
+		private function isItemVisible($item) {
+			if ($this->env->authentication()->isAdmin()) return TRUE;
+			$permission = $this->getItemUserPermissionFromCache($item);
+			if (strcmp(Authentication::PERMISSION_VALUE_NO_RIGHTS, $permission) != 0) return TRUE;
+			return FALSE;
 		}
 
 		public function details($item) {
@@ -171,7 +196,8 @@
 			$details = $item->details();
 			$details["description"] = $this->description($item);
 			$details["permission"] = $this->permission($item);
-			foreach($this->providers as $p) {
+			
+			foreach($this->detailsPlugins as $p) {
 				$l = $p->getItemDetails($item, $details);
 				if (!$l) continue;
 				
@@ -207,6 +233,15 @@
 			return $permission;
 		}
 		
+		public function fetchPermissions($folder) {
+			if ($this->env->authentication()->isAdmin()) return;
+			$permissions = $this->env->configuration()->getAllItemPermissions($folder, $this->env->authentication()->getUserId());
+
+			$this->permissionCacheFolders[] = $folder->id();
+			foreach($permissions as $id => $p)
+				$this->permissionCache[$id] = $p;
+		}
+		
 		public function temporaryItemPermission($item, $permission) {
 			$this->permissionCache[$item->id()] = $permission;
 		}
@@ -217,12 +252,30 @@
 				Logging::logDebug("Permission cache get [".$item->id()."]=".$permission);
 			} else {
 				$permission = $this->env->configuration()->getItemPermission($item, $this->env->authentication()->getUserId());
+				if (!$permission) return $this->env->authentication()->getDefaultPermission();
+				
 				$this->permissionCache[$item->id()] = $permission;
 				Logging::logDebug("Permission cache put [".$item->id()."]=".$permission);
 			}
 			return $permission;
 		}
 
+		private function getItemUserPermissionFromCache($item) {
+			if (array_key_exists($item->id(), $this->permissionCache)) {
+				$permission = $this->permissionCache[$item->id()];
+				Logging::logDebug("Permission cache get [".$item->id()."]=".$permission);
+			} else {
+				$parentId = $item->parent()->id();
+				if ($item->isFile() and array_key_exists($parentId, $this->permissionCache)) {
+					$permission = $this->permissionCache[$parentId];
+					Logging::logDebug("Permission cache get [".$item->id()."->".$parentId."]=".$permission);
+				} else {
+					return $this->env->authentication()->getDefaultPermission();
+				}
+			}
+			return $permission;
+		}
+		
 		public function allPermissions($item) {
 			return $this->env->configuration()->getItemPermissions($item);
 		}
@@ -255,20 +308,26 @@
 		public function copy($item, $to) {
 			Logging::logDebug('copying '.$item->id()."[".$item->path().'] to ['.$to.']');
 			
-			if ($to->isFile()) throw new ServiceException("NOT_A_DIR", $to->path());
+			if (!$item->isFile() and $to->isFile()) throw new ServiceException("NOT_A_DIR", $to->path());
+			if ($item->isFile() and !$to->isFile()) throw new ServiceException("NOT_A_FILE", $to->path());
+			
 			$this->assertRights($item, Authentication::RIGHTS_READ, "copy");
-			$this->assertRights($to, Authentication::RIGHTS_WRITE, "copy");
+			$this->assertRights($to->parent(), Authentication::RIGHTS_WRITE, "copy");
 
 			$to = $item->copy($to);
 			$this->env->events()->onEvent(FileEvent::copy($item, $to));
 		}
 		
-		public function copyItems($items, $to) {
-			Logging::logDebug('copying '.count($items).' items');
-			$this->assertRights($items, Authentication::RIGHTS_WRITE, "copy");
+		public function copyItems($items, $folder) {
+			Logging::logDebug('copying '.count($items).' items to ['.$to.']');
+			$this->assertRights($items, Authentication::RIGHTS_READ, "copy");
 			
-			foreach($items as $item)
-				$this->copy($item, $to);
+			foreach($items as $item) {
+				if ($item->isFile)
+					$this->copy($item, $folder->fileWithName($item->name(), TRUE));
+				else
+					$this->copy($item, $folder->folderWithName($item->name(), TRUE));
+			}
 		}
 		
 		public function move($item, $to) {
@@ -329,6 +388,9 @@
 
 			$new = $parent->createFolder($name);
 			$this->env->events()->onEvent(FileEvent::createFolder($new));
+			
+			if ($this->env->features()->isFeatureEnabled("permission_update") and !$this->env->authentication()->isAdmin())
+				$this->env->configuration()->addItemPermission($new->id(), Authentication::PERMISSION_VALUE_READWRITE, $this->env->authentication()->getUserId());
 		}
 
 		public function download($file, $range = NULL) {
@@ -365,9 +427,16 @@
 			if (!$range)
 				$this->env->events()->onEvent(FileEvent::download($file));
 
-			$this->env->response()->download($name, $file->read($range), $size, $range);							
+			$this->env->response()->download($name, $file->extension(), $file->read($range), $size, $range);							
 		}
 
+		public function view($file) {
+			Logging::logDebug('view ['.$file->id().']');
+			$this->assertRights($file, Authentication::RIGHTS_READ, "view");
+			$this->env->events()->onEvent(FileEvent::download($file));
+			$this->env->response()->send($file->name(), $file->extension(), $file->read(), $file->size());
+		}
+		
 		public function uploadTo($folder) {
 			$this->env->features()->assertFeature("file_upload");
 			$this->assertRights($folder, Authentication::RIGHTS_WRITE, "upload");
@@ -401,7 +470,7 @@
 		}
 		
 		private function upload($folder, $name, $origin) {
-			$target = $folder->createEmptyItem($name);
+			$target = $folder->createFile($name);
 			Logging::logDebug('uploading to ['.$target.']');
 			
 			$src = @fopen($origin, "r");
@@ -456,6 +525,10 @@
 			Logging::logDebug("FILESYSTEM: allowed_file_upload_types=".Util::array2str($this->allowedUploadTypes));
 		}
 
+		public function __toString() {
+			return "FILESYSTEMCONTROLLER";
+		}
+
 	 }
 	 
 	 class FileEvent extends Event {
@@ -523,14 +596,19 @@
 		}
 				
 		public function details() {
-			$f = $this->item->internalId()." (".$this->item->filesystem()->name().")";
+			$f = $this->item->id()." (".$this->item->filesystem()->name().")";
 			
 			if ($this->subType() === self::RENAME)
 				return 'item id='.$f.';to='.$this->info;
 			if ($this->subType() === self::COPY or $this->subType() === self::MOVE)
-				return 'item id='.$f.';to='.$this->info->internalId()." (".$this->info->filesystem()->name().")";
+				return 'item id='.$f.';to='.$this->info->id()." (".$this->info->filesystem()->name().")";
 			return 'item id='.$f;
 		}
+		
+		public function __toString() {
+			return "FILESYSTEMEVENT ".get_class($this);
+		}
+
 	}
 
 ?>
\ No newline at end of file
